@@ -3,7 +3,7 @@ package Net::BitTorrent;
     use 5.010;
     use Moose;
     use Moose::Util::TypeConstraints;
-    our $MAJOR = 0.074; our $MINOR = 0; our $DEV = 4; our $VERSION = sprintf('%1.3f%03d' . ($DEV ? (($DEV < 0 ? '' : '_') . '%03d') : ('')), $MAJOR, $MINOR, abs $DEV);
+    our $MAJOR = 0.074; our $MINOR = 0; our $DEV = 5; our $VERSION = sprintf('%1.3f%03d' . ($DEV ? (($DEV < 0 ? '' : '_') . '%03d') : ('')), $MAJOR, $MINOR, abs $DEV);
     use AnyEvent;
     use lib '../../lib';
     use Net::BitTorrent::Types qw[:client];
@@ -98,7 +98,8 @@ package Net::BitTorrent;
     #
     has 'ip_filter' => (is       => 'ro',
                         isa      => 'Net::BitTorrent::Network::IPFilter',
-                        init_arg => undef
+                        init_arg => undef,
+                        builder  => '_build_ip_filter'
     );
 
     sub _build_ip_filter {
@@ -138,7 +139,7 @@ package Net::BitTorrent;
                                writer     => '_set_' . $prot . $ipv,
                                predicate  => '_has_' . $prot . $ipv
                     );
-                has $prot 
+                has $prot
                     . $ipv
                     . '_sock' => (
                                  is         => 'ro',
@@ -149,7 +150,7 @@ package Net::BitTorrent;
                                  writer => '_set_' . $prot . $ipv . '_sock',
                                  predicate => '_has_' . $prot . $ipv . '_sock'
                     );
-                has $prot 
+                has $prot
                     . $ipv
                     . '_host' => (
                                  is      => 'ro',
@@ -165,7 +166,7 @@ package Net::BitTorrent;
 
     sub _build_tcp6 {
         my $s = shift;
-        my $server;
+        my ($server, $actual_socket, $actual_host, $actual_port);
         for my $port (ref $s->port ? @{$s->port} : $s->port) {
             require Net::BitTorrent::Network::Utility;
             $server = Net::BitTorrent::Network::Utility::server(
@@ -173,7 +174,7 @@ package Net::BitTorrent;
                 $port,
                 sub { $s->_on_tcp6_in(@_); },
                 sub {
-                    my ($actual_socket, $actual_host, $actual_port) = @_;
+                    ($actual_socket, $actual_host, $actual_port) = @_;
 
                     #if ($self->port != $port) { ...; }
                     $s->_set_tcp6_sock($actual_socket);
@@ -184,20 +185,34 @@ package Net::BitTorrent;
             );
             last if defined $server;
         }
-        return $server if $server;
-        $s->trigger_listen_failed(
-            {protocol => 'tcp6',
-             severity => 'fatal',
-             event    => 'listen_failed',
-             message => 'Failed to open IPv6 port to the outside world: ' . $!
-            }
-        );
-        return;
+        if ($server) {
+            $s->trigger_listen_success(
+                              {port     => $actual_port,
+                               protocol => 'tcp6',
+                               severity => 'debug',
+                               event    => 'listen_success',
+                               message  => sprintf
+                                   'Opened IPv6 port %d to the outside world',
+                               $actual_port
+                              }
+            );
+        }
+        else {
+            $s->trigger_listen_failure(
+                 {protocol => 'tcp6',
+                  severity => 'fatal',
+                  event    => 'listen_failure',
+                  message => 'Failed to open IPv6 port to the outside world: '
+                      . $!
+                 }
+            );
+        }
+        return $server;
     }
 
     sub _build_tcp4 {
         my $s = shift;
-        my $server;
+        my ($server, $actual_socket, $actual_host, $actual_port);
         for my $port (ref $s->port ? @{$s->port} : $s->port) {
             require Net::BitTorrent::Network::Utility;
             $server = Net::BitTorrent::Network::Utility::server(
@@ -205,7 +220,7 @@ package Net::BitTorrent;
                 $port,
                 sub { $s->_on_tcp4_in(@_); },
                 sub {
-                    my ($actual_socket, $actual_host, $actual_port) = @_;
+                    ($actual_socket, $actual_host, $actual_port) = @_;
 
                     #if ($self->port != $port) { ...; }
                     $s->_set_tcp4_sock($actual_socket);
@@ -216,20 +231,34 @@ package Net::BitTorrent;
             );
             last if defined $server;
         }
-        return $server if $server;
-        $s->trigger_listen_failed(
-            {protocol => 'tcp4',
-             severity => 'fatal',
-             event    => 'listen_failed',
-             message => 'Failed to open IPv4 port to the outside world: ' . $!
-            }
-        );
-        return;
+        if ($server) {
+            $s->trigger_listen_success(
+                              {port     => $actual_port,
+                               protocol => 'tcp4',
+                               severity => 'debug',
+                               event    => 'listen_success',
+                               message  => sprintf
+                                   'Opened IPv4 port %d to the outside world',
+                               $actual_port
+                              }
+            );
+        }
+        else {
+            $s->trigger_listen_failure(
+                 {protocol => 'tcp4',
+                  severity => 'fatal',
+                  event    => 'listen_failure',
+                  message => 'Failed to open IPv4 port to the outside world: '
+                      . $!
+                 }
+            );
+        }
+        return $server;
     }
 
     sub _build_udp6 {
         my $s = shift;
-        my $server;
+        my ($server, $actual_socket, $actual_host, $actual_port);
         for my $port (ref $s->port ? @{$s->port} : $s->port) {
             require Net::BitTorrent::Network::Utility;
             $server = Net::BitTorrent::Network::Utility::server(
@@ -237,7 +266,7 @@ package Net::BitTorrent;
                 $port,
                 sub { $s->_on_udp6_in(@_); },
                 sub {
-                    my ($actual_socket, $actual_host, $actual_port) = @_;
+                    ($actual_socket, $actual_host, $actual_port) = @_;
 
                     #if ($self->port != $port) { ...; }
                     $s->_set_udp6_sock($actual_socket);
@@ -248,20 +277,34 @@ package Net::BitTorrent;
             );
             last if defined $server;
         }
-        return $server if $server;
-        $s->trigger_listen_failed(
-            {protocol => 'udp6',
-             severity => 'fatal',
-             event    => 'listen_failed',
-             message => 'Failed to open IPv6 port to the outside world: ' . $!
-            }
-        );
-        return;
+        if ($server) {
+            $s->trigger_listen_success(
+                              {port     => $actual_port,
+                               protocol => 'udp6',
+                               severity => 'debug',
+                               event    => 'listen_success',
+                               message  => sprintf
+                                   'Opened IPv6 port %d to the outside world',
+                               $actual_port
+                              }
+            );
+        }
+        else {
+            $s->trigger_listen_failure(
+                 {protocol => 'udp6',
+                  severity => 'fatal',
+                  event    => 'listen_failure',
+                  message => 'Failed to open IPv6 port to the outside world: '
+                      . $!
+                 }
+            );
+        }
+        return $server;
     }
 
     sub _build_udp4 {
         my $s = shift;
-        my $server;
+        my ($server, $actual_socket, $actual_host, $actual_port);
         for my $port (ref $s->port ? @{$s->port} : $s->port) {
             require Net::BitTorrent::Network::Utility;
             $server = Net::BitTorrent::Network::Utility::server(
@@ -269,7 +312,7 @@ package Net::BitTorrent;
                 $port,
                 sub { $s->_on_udp4_in(@_); },
                 sub {
-                    my ($actual_socket, $actual_host, $actual_port) = @_;
+                    ($actual_socket, $actual_host, $actual_port) = @_;
 
                     #if ($self->port != $port) { ...; }
                     $s->_set_udp4_sock($actual_socket);
@@ -280,27 +323,42 @@ package Net::BitTorrent;
             );
             last if defined $server;
         }
-        return $server if $server;
-        $s->trigger_listen_failed(
-            {protocol => 'udp4',
-             severity => 'fatal',
-             event    => 'listen_failed',
-             message => 'Failed to open IPv4 port to the outside world: ' . $!
-            }
-        );
-        return;
+        if ($server) {
+            $s->trigger_listen_success(
+                              {port     => $actual_port,
+                               protocol => 'udp4',
+                               severity => 'debug',
+                               event    => 'listen_success',
+                               message  => sprintf
+                                   'Opened IPv4 port %d to the outside world',
+                               $actual_port
+                              }
+            );
+        }
+        else {
+            $s->trigger_listen_failure(
+                 {protocol => 'udp4',
+                  severity => 'fatal',
+                  event    => 'listen_failure',
+                  message => 'Failed to open IPv4 port to the outside world: '
+                      . $!
+                 }
+            );
+        }
+        return $server;
     }
 
     sub _on_tcp4_in {
         my ($self, $peer, $paddr, $host, $port) = @_;
-        my $range = $self->ip_filter->is_banned($host);
-        if (defined $range) {
+        my $rule = $self->ip_filter->is_banned($host);
+        if (defined $rule) {
             $self->trigger_ip_filter(
                      {protocol => 'tcp4',
                       severity => 'debug',
                       event    => 'ip_filter',
                       ip       => $host,
-                      range    => $range,
+                      port     => $port,
+                      rule     => $rule,
                       message => 'Incoming connection was blocked by ipfilter'
                      }
             );
@@ -314,14 +372,14 @@ package Net::BitTorrent;
 
     sub _on_tcp6_in {
         my ($self, $peer, $paddr, $host, $port) = @_;
-        my $range = $self->ip_filter->is_banned($host);
-        if (defined $range) {
+        my $rule = $self->ip_filter->is_banned($host);
+        if (defined $rule) {
             $self->trigger_ip_filter(
                      {protocol => 'tcp4',
                       severity => 'debug',
                       event    => 'ip_filter',
                       ip       => $host,
-                      range    => $range,
+                      rule     => $rule,
                       message => 'Incoming connection was blocked by ipfilter'
                      }
             );
@@ -336,14 +394,14 @@ package Net::BitTorrent;
     sub _on_udp4_in {
         my $s = shift;
         my ($udp, $sock, $paddr, $host, $port, $data, $flags) = @_;
-        my $range = $s->ip_filter->is_banned($host);
-        if (defined $range) {
+        my $rule = $s->ip_filter->is_banned($host);
+        if (defined $rule) {
             $s->trigger_ip_filter(
                            {protocol => 'udp4',
                             severity => 'debug',
                             event    => 'ip_filter',
                             ip       => $host,
-                            range    => $range,
+                            rule     => $rule,
                             message => 'Incoming data was blocked by ipfilter'
                            }
             );
@@ -355,14 +413,14 @@ package Net::BitTorrent;
     sub _on_upd6_in {
         my $s = shift;
         my ($udp, $sock, $paddr, $host, $port, $data, $flags) = @_;
-        my $range = $s->ip_filter->is_banned($host);
-        if (defined $range) {
+        my $rule = $s->ip_filter->is_banned($host);
+        if (defined $rule) {
             $s->trigger_ip_filter(
                            {protocol => 'upd6',
                             severity => 'debug',
                             event    => 'ip_filter',
                             ip       => $host,
-                            range    => $range,
+                            rule     => $rule,
                             message => 'Incoming data was blocked by ipfilter'
                            }
             );
@@ -393,7 +451,7 @@ package Net::BitTorrent;
         my ($code, $self, $arg) = @_;
         blessed $arg ? $code->($self, $arg->_id, $arg) : $code->($self, $arg);
     };
-    if (0) {                                            # Callback system
+    if (1) {                                            # Callback system
 
         sub _build_callback_no_op {
             sub {1}
@@ -404,13 +462,14 @@ package Net::BitTorrent;
                         handles    => {"trigger_$_" => 'execute_method'},
                         lazy_build => 1,
                         builder    => '_build_callback_no_op',
+                        clearer    => "_no_$_",
                         weak_ref   => 1
             )
             for qw[
-            peer_construction peer_destruction
+            listen_failure listen_success
         ];
     }
-    {    # Callback System II
+    else {    # Callback System II
         for my $type (qw[peer_construction peer_destruction]) {
             has "_${type}_callbacks" => (
                                    isa      => 'ArrayRef[Ref]',
@@ -541,6 +600,6 @@ L<clarification of the CCA-SA3.0|http://creativecommons.org/licenses/by-sa/3.0/u
 Neither this module nor the L<Author|/Author> is affiliated with BitTorrent,
 Inc.
 
-=for rcs $Id: BitTorrent.pm 5d05177 2010-07-03 18:25:20Z sanko@cpan.org $
+=for rcs $Id: BitTorrent.pm 4c24394 2010-07-05 02:43:04Z sanko@cpan.org $
 
 =cut
